#include <assert.h>
#include <math.h>
#include <ostream>

template<class T> static inline bool very_close(T a, T b) { return a == b; }
template<> inline bool very_close(double a, double b) { return fabs(a-b) < 0.0000000000001; }
template<> inline bool very_close(float a, float b) { return fabs(a-b) < 0.00000001; }

extern "C" {
int nnls_c(double* a, const int* mda, const int* m, const int* n, double* b, 
	   double* x, double* rnorm, double* w, double* zz, int* index, 
	   int* mode);
};

template<class T> class Matrix {
  T *data;
  int *refs;

 public:
  int xsize, ysize;

  Matrix() : 
    xsize(0),
    ysize(0),
    data(0),
    refs(0) {
  }

  Matrix(int w, int h) :
    xsize(w),
    ysize(h),
    data(new T[w * h]),
    refs(new int(1)) {
    zero();
  }

  // square unity matrix
  explicit Matrix(int size) : 
    xsize(size),
    ysize(size),
    data(new T[size * size]),
    refs(new int(1)) {
      fprintf(stderr, "UNITY\n");
      unity(1);
  }

  Matrix(const Matrix &x) :
    xsize(x.xsize),
    ysize(x.ysize),
    data(x.data),
    refs(x.refs) {
    if(refs) (*refs)++;
  }

  void free() {
    if (refs) {
      if (0 == --(*refs)) {
	delete[] data;
	delete refs;
	data = 0;
	refs = 0;
      }
    }
  }

  void Reshape(int x, int y) {
    assert(x * y == xsize * ysize);
    xsize = x;
    ysize = y;
  }

  Matrix& operator=(const Matrix &x) {
    if(&x != this) {
      free();
      xsize = x.xsize;
      ysize = x.ysize;
      data = x.data;
      refs = x.refs;
      if(refs) (*refs)++;
    }
    return *this;
  }

  void Modify() {
    if (refs && *refs > 1) {
      (*refs)--;
      refs = new int(1);
      T *x = new T[xsize * ysize];
      for(int e=0;e<xsize*ysize;e++) x[e] = data[e];
      data = x;
    }
  }

  Matrix& operator=(const T &x) {
    Modify();
    for(int e=0;e<xsize*ysize;e++) data[e]=x;
    return *this;
  }

  ~Matrix() { free(); }

  void zero() {
    Modify();
    for(int e=0;e<xsize*ysize;e++) data[e]=0;
  }

  void unity(T val) {
    Modify();
    assert(xsize == ysize);
    zero();
    for(int e=0;e<xsize;e++)
      data[e * xsize + e]=val;
  }

  void operator+=(T val) {
    Modify();
    for(int e=0;e<xsize*ysize;e++)
      data[e] += val;
  }

  Matrix operator*(T val) {
    Matrix ret(xsize, ysize);
    for(int e=0;e<xsize*ysize;e++)
      ret.data[e] += data[e] * val;
    return ret;
  }

  void operator+=(const Matrix<T> &mat) {
    Modify();
    assert(mat.xsize == xsize);
    assert(mat.ysize == ysize);
    for(int e=0;e<xsize*ysize;e++)
      data[e] += mat.data[e];
  }

  Matrix operator-(const Matrix<T> &mat) const {
    assert(mat.xsize == xsize);
    assert(mat.ysize == ysize);
    Matrix ret(xsize, ysize);
    for(int e=0;e<xsize*ysize;e++)
      ret.data[e] = data[e] - mat.data[e];
    return ret;
  }

  T *row(int y) {
    Modify();
    assert(y>=0);
    assert(y<ysize);
    return data + y * xsize;
  }

  const T &value(int x, int y) const {
    assert(x>=0);
    assert(y>=0);
    assert(x<xsize);
    assert(y<ysize);
    return data[y * xsize + x];
  }

  T &value(int x, int y) {
    Modify();
    assert(x>=0);
    assert(y>=0);
    assert(x<xsize);
    assert(y<ysize);
    return data[y * xsize + x];
  }

  void swaprows(int a, int b) {
    Modify();
    assert(a!=b);
    assert(a>=0);
    assert(b>=0);
    assert(a<ysize);
    assert(b<ysize);
    T *ap = data + a * xsize;
    T *bp = data + b * xsize;
    for(int e=0;e<xsize;e++) {
      T tmp = *ap;
      *ap = *bp;
      *bp = tmp;
      ap++;
      bp++;
    }
  }

  void rowmulsub(int start, int a, int b, T val) {
    Modify();
    assert(a!=b);
    assert(a>=0);
    assert(b>=0);
    assert(a<ysize);
    assert(b<ysize);
    T *ap = data + a * xsize + start;
    T *bp = data + b * xsize + start; 
    for(int e=start;e<xsize;e++) *(ap++) -= *(bp++) * val;
  }

  void mulrow(int a, T val) {
    Modify();
    assert(a>=0);
    assert(a<ysize);
    T *ap = data + a * xsize;
    for(int e=0;e<xsize;e++) *(ap++) *= val;
  }

  bool solve(Matrix<T> &answers) {
    Modify();
    assert(xsize == ysize);
    assert(ysize == answers.ysize);
    for(int x=0;x<xsize;x++)
    {
      for(int y=0;y<ysize;y++)
      {
	if(x==y) continue;
	if(value(x,y) == 0.0) continue;
	if(value(x,x) == 0.0) {
	  answers.swaprows(x,y);
	  swaprows(x,y);
	} else {
	  T tmp = value(x,y)/value(x,x);
	  answers.rowmulsub(0, y, x, tmp);
	  rowmulsub(x, y, x, tmp);
	  value(x,y) = 0; 
	}
      }
    }
    for(int x=0;x<xsize;x++)
    {
      if(value(x,x) == 0.0) return false;
      T tmp = 1.0 / value(x,x);
      answers.mulrow(x, tmp);
      // mulrow(x, tmp);
    }
    return true;
  }

  Matrix solve_least_square(const Matrix<T> &answers) const {
    fprintf(stderr, "start\n");
    assert(answers.ysize == ysize);
    Matrix tmp1 = this->transpose();
    fprintf(stderr, "transpose\n");
    Matrix tmp2 = tmp1 * *this;
    fprintf(stderr, "mult1\n");
    Matrix tmp3 = tmp1 * answers;
    fprintf(stderr, "mult2\n");
    tmp2.solve(tmp3);
    fprintf(stderr, "done\n");
    return tmp3;
  }

  Matrix invert() const {
    fprintf(stderr, "start\n");
    Matrix tmp1 = this->transpose();
    fprintf(stderr, "transpose\n");
    Matrix tmp2 = tmp1 * *this;
    fprintf(stderr, "mult1\n");
    tmp2.solve(tmp1);
    fprintf(stderr, "done\n");
    return tmp1;
  }

  Matrix solve_nnls(const Matrix<T> &answers) const {
    assert(answers.xsize == 1);
    assert(answers.ysize == ysize);
    int M = ysize;
    int N = xsize;
    int mda = M;
    double rnorm;
    double *A = new double[M * N];
    double *B = new double[M];
    double *X = new double[N];
    double *w = new double[N];
    double *zz = new double[M];
    int *index = new int[N];
    int mode;
    for (int y = 0; y < ysize; y++) {
      for (int x = 0; x < xsize; x++) {
	A[x * mda + y] = value(x, y);
      }
    }
    for (int i = 0; i < M; i++) {
      B[i] = answers.data[i];
    }

    nnls_c(A,
	   &mda,
	   &M,
	   &N,
	   B,
	   X,
	   &rnorm,
	   w,
	   zz,
	   index,
	   &mode);
    delete[] A;
    delete[] B;
    delete[] w;
    delete[] zz;
    delete[] index;
    Matrix ret(1, N);
    for (int i = 0; i < N; i++) {
      ret.data[i] = X[i];
    }
    delete[] X;
    assert(mode == 1);
    return ret;
  }
  
  Matrix invert_nnls() {
    Matrix<float> ans(1, ysize);
    Matrix<float> ret(ysize, xsize);
    for (int x = 0; x < ysize; x++) {
      fprintf(stderr, "invert %d/%d\n", x, ysize);
      if (x > 0) ans.value(0, x - 1) = 0.0;
      ans.value(0, x) = 1.0;
      Matrix<float> tmp = solve_nnls(ans);
      for (int y = 0; y < tmp.ysize; y++) {
	ret.value(x, y) = tmp.value(0, y);
      }
    }
    return ret;
  }

  Matrix transpose() const {
    Matrix ret(ysize, xsize);
    for(int y=0;y<ysize;y++)
      for(int x=0;x<xsize;x++)
	ret.value(y,x) = value(x,y);

    return ret;
  }

  Matrix operator*(const Matrix &m) const {
    assert(xsize == m.ysize);
    Matrix ret(m.xsize, ysize);
    for(int z=0;z<xsize;z++) {
      for(int y=0;y<ysize;y++) {
	if(value(z,y) != 0.0) {
	  for(int x=0;x<m.xsize;x++) {
	    ret.value(x,y) += value(z,y) * m.value(x,z);
	  }
	}
      }
    }
    return ret;
  }

  bool operator==(const Matrix &m) const {
    if(m.data == data) return true;
    if(m.xsize != xsize || m.ysize != ysize)
      return false;
    for(int y=0;y<ysize;y++)
      for(int x=0;x<xsize;x++)
	if( ! very_close<T>(m.value(x,y), value(x,y)))
	  return false;
    return true;
  }

  double zp() {
    Modify();
    int zc = 0;
    for(int y=0;y<ysize;y++)
      for(int x=0;x<xsize;x++)
	if(value(x,y) == 0.0)
	  zc++;
    return zc/(double)(xsize * ysize);
  }

  void MinZero() {
    Modify();
    for(int e=0;e<xsize*ysize;e++)
      if (data[e] < 0)
	data[e] = 0;
  }
};

template<class T> std::ostream& operator<<(std::ostream& os, const Matrix<T> m) {
  os<<std::endl;
  for(int y = 0; y<m.ysize;y++) {
    os << (y == 0 ? "/ " : y == m.ysize - 1 ? "\\ " : "| ");  
    for(int x = 0; x<m.xsize;x++) {
      if(x != 0) os << ", ";
      os << m.value(x,y);
    }
    os << (y == 0 ? " \\\n" : y == m.ysize - 1 ? " /\n" : " |\n");  
  }
  return os;
}
